import { CreateProposal, UpdateBDIdentification } from '@agiliza/api/domain'
import { ClienteApi, Configuration } from '@microcredito/client'

import { ErrorMappers, ProposalDataApiMappers, ProposalDataMappers } from '../../mappers'
import { API_URL, mapUserAgentToString, SESSION_KEY, UserAgent } from '../shared'

export interface PDCustomerDataDataRepository {
  createProposal(input: CreateProposal['Input']): Promise<CreateProposal['Output']>
  updateIdentification(input: UpdateBDIdentification['Input']): Promise<UpdateBDIdentification['Output']>
}

export class PDCustomerDataRepositoryImpl implements PDCustomerDataDataRepository {
  private api: ClienteApi
  private errorAdapter: ErrorMappers.ResponseErrorAdapter
  private updateIdentificationApiMapper: ProposalDataApiMappers.BDUpdateIdentificationApiMapper
  private identificationMapper: ProposalDataMappers.BDIdentificationMapper
  private createProposalApiMapper: ProposalDataApiMappers.CreateProposalApiMapper

  constructor(userAgent: string, accessToken: string) {
    this.errorAdapter = new ErrorMappers.ResponseErrorAdapter()
    this.updateIdentificationApiMapper = new ProposalDataApiMappers.BDUpdateIdentificationApiMapper()
    this.identificationMapper = new ProposalDataMappers.BDIdentificationMapper()
    this.createProposalApiMapper = new ProposalDataApiMappers.CreateProposalApiMapper()

    this.api = new ClienteApi(
      new Configuration({
        basePath: API_URL,
        accessToken,
        headers: {
          'User-Agent': userAgent,
        },
      })
    )
  }

  public updateIdentification = async (input: UpdateBDIdentification['Input']): Promise<UpdateBDIdentification['Output']> => {
    try {
      const response = await this.api.atualizarIdentificacaoDadosNegocio(this.updateIdentificationApiMapper.mapDomainToApiModel(input))
      return this.identificationMapper.mapApiModelToDomain(response)
    } catch (e) {
      const result = await this.errorAdapter.mapApiModelToDomain(e)
      throw result
    }
  }

  public createProposal = async (input: CreateProposal['Input']): Promise<CreateProposal['Output']> => {
    try {
      const response = await this.api.criarProposta(this.createProposalApiMapper.mapDomainToApiModel(input))
      return { proposalId: response.propostaId || '' }
    } catch (e) {
      const result = await this.errorAdapter.mapApiModelToDomain(e)
      throw result
    }
  }
}

export class PDCustomerDataRepositoryImplFactory {
  static create(userAgent: UserAgent) {
    const accessToken = localStorage.getItem(SESSION_KEY) || ''
    const repository = new PDCustomerDataRepositoryImpl(mapUserAgentToString(userAgent), accessToken)
    return repository
  }
}
