import React, { useMemo, useState } from 'react'
import { RouteComponentProps } from 'react-router'

import { useFormState } from '@agiliza/utils/hooks/state'
import { Button, MobileStepper, Typography } from '@material-ui/core'
import { withStyles, WithStyles } from '@material-ui/core/styles'
import {
  AccountBox as AccountBoxIcon,
  CheckBox as CheckBoxIcon,
  ContactMail as ContactMailIcon,
  DriveEta as DriveEtaIcon,
  Forum as ForumIcon,
  Home as HomeIcon,
  KeyboardArrowLeft as KeyboardArrowLeftIcon,
  KeyboardArrowRight as KeyboardArrowRightIcon,
  List as ListIcon,
  LocalAtm as LocalAtmIcon,
  ZoomOutMap as ZoomOutMapIcon
} from '@material-ui/icons'

import { businessDataItems } from './BusinessData'
import { connected, ConnectedProps } from './connect'
import Address from './CustomerData/Address'
import Home from './CustomerData/Home'
import Identification, { initialState as cdIdentInitState } from './CustomerData/Identification'
import PersonalData from './CustomerData/PersonalData'
import References from './CustomerData/References'
import SourceIncome from './CustomerData/SourceIncome'
import SpouseAdditionalData from './CustomerData/SpouseAdditionalData'
import SpouseData from './CustomerData/SpouseData'
import Vehicles from './CustomerData/Vehicles'
import styles from './shared-styles'

export const PROPOSAL_DATA_PATHS = {
  selectOption: '/selecionarOpcao',
  involvedPeople: '/pessoasEnvolvidas',
  businessData: '/dadosNegocio',
}

export interface ProposalDataRouteState {
  subproductId: string
}

type ExtendedProps = WithStyles<typeof styles> & RouteComponentProps & ConnectedProps

interface Props extends ExtendedProps {}

const ProposalData = (props: Props) => {
  const { classes } = props
  const [activeStep, setActiveStep] = useState(0)

  const cdIdentification = useFormState('proposalDataState', cdIdentInitState)

  const customerDataItems = useMemo(
    () => [
      {
        label: 'Identificação',
        icon: <AccountBoxIcon />,
        component: <Identification state={cdIdentification.state} actions={cdIdentification.actions} />,
      },
      { label: 'Dados Pessoais', icon: <ListIcon />, component: <PersonalData /> },
      { label: 'Endereço', icon: <ContactMailIcon />, component: <Address /> },
      { label: 'Fonte de renda', icon: <LocalAtmIcon />, component: <SourceIncome /> },
      { label: 'Cônjuge - Ident.', icon: <ForumIcon />, component: <SpouseData /> },
      { label: 'Cônjuge +', icon: <ZoomOutMapIcon />, component: <SpouseAdditionalData /> },
      { label: 'Referências', icon: <CheckBoxIcon />, component: <References /> },
      { label: 'Moradia', icon: <HomeIcon />, component: <Home /> },
      { label: 'Veículos', icon: <DriveEtaIcon />, component: <Vehicles /> },
    ],
    [cdIdentification]
  )

  const proposalDataItems = useMemo(() => [...customerDataItems, ...businessDataItems], [customerDataItems, businessDataItems])

  const steps = useMemo(() => proposalDataItems.length, [proposalDataItems])
  // const location = useLocation<ProposalDataRouteState>()

  const handleNext = () => {
    // if (activeStep === steps - 1) history.push({ pathname: PATHS.creditLines })
    setActiveStep((prevActiveStep) => prevActiveStep + 1)
  }

  const handleBack = () => {
    setActiveStep((prevActiveStep) => prevActiveStep - 1)
  }

  const mapActiveStepToTitle = () => {
    if (activeStep < customerDataItems.length) return 'Dados do cliente'
    if (activeStep < proposalDataItems.length) return 'Dados de negócio'
  }

  return (
    <div className={classes.pageContent}>
      <Typography className={classes.pageTitle} variant="h5">
        {mapActiveStepToTitle()}
      </Typography>
      <Typography className={classes.pageSubtitle}>{proposalDataItems[activeStep].label}</Typography>
      <div className={classes.stepContentContainer}>{proposalDataItems[activeStep].component}</div>
      <div className={classes.stepperContainer}>
        {/* <Stepper activeStep={activeStep} alternativeLabel className={classes.stepper}> */}
        {/* {propostalDataItems.map((iPI, i) => (
            <Step key={i}>
              <StepLabel>{iPI.label}</StepLabel>
            </Step>
          ))} */}
        <MobileStepper
          variant="dots"
          steps={steps}
          position="static"
          activeStep={activeStep}
          style={{ width: '100%' }}
          nextButton={
            <Button size="large" onClick={handleNext} disabled={activeStep === steps - 1} className={classes.stepperBtn}>
              {activeStep === steps - 1 ? (
                'Finalizar'
              ) : (
                <>
                  Próximo
                  <KeyboardArrowRightIcon />
                </>
              )}
            </Button>
          }
          backButton={
            <Button size="large" onClick={handleBack} disabled={activeStep === 0} className={classes.stepperBtn}>
              <KeyboardArrowLeftIcon />
              Anterior
            </Button>
          }
        />
        {/* </Stepper> */}
      </div>
    </div>
  )
}

export default connected(withStyles(styles)(ProposalData))
